@@ -81,12 +81,15 @@ class ActSet():
     def get_ignore_class_index(self) -> Union[int, Tuple]:
         """Return the index of ignore classes, i.e., ``is_ignore=True``.
         If there are only one ignore class, return the index as int.
+        When there is no ignore classe, return None.
 
         Returns:
-            Union[int, Tuple]
+            Union[int, Tuple] or None
         """
         index = tuple([i for i, act in enumerate(
             self.classes) if act.is_ignore])
+        if len(index) == 0:
+            return None
         if len(index) == 1:
             return index[0]
         return index
@@ -94,11 +97,14 @@ class ActSet():
     def get_ignore_class_id(self) -> Union[int, Tuple]:
         """Return the ID of ignore classes, i.e., ``is_ignore=True``.
         If there are only one ignore class, return the ID as int.
+        When there is no ignore classe, return None.
 
         Returns:
-            Union[int, Tuple]
+            Union[int, Tuple] or None
         """
         ids = tuple([act.id for act in self.classes if act.is_ignore])
+        if len(ids) == 0:
+            return None
         if len(ids) == 1:
             return ids[0]
         return ids
@@ -112,7 +118,10 @@ class ActSet():
         Returns:
             np.ndarray: 1d array of activity index.
         """
-        assert set(ids.ravel()) <= set(self.get_ids())
+        assert set(ids.ravel()) <= set(self.get_ids()), (
+            f"expected ID set is {set(self.get_ids())}, "
+            f"but got ID set={set(ids.ravel())}."
+        )
 
         index = np.full(ids.shape, -1)
         for i, cls in enumerate(self.classes):
@@ -175,20 +184,3 @@ class ActSet():
             if cls.name == cls_name:
                 return cls.id
         raise ValueError(f"got unexpected class name. cls_name={cls_name}")
-
-
-""" Activity Set Definitions
-"""
-
-OPENPACK_OPERATIONS = ActSet((
-    ActClass(100, "Picking"),
-    ActClass(200, "MoveItemLabel"),
-    ActClass(300, "AssembleBox"),
-    ActClass(400, "PackInBox"),
-    ActClass(500, "CloseBox"),
-    ActClass(600, "ReadLabel"),
-    ActClass(700, "AttachLabel"),
-    ActClass(800, "PutOnCartRack"),
-    ActClass(900, "Pen"),
-    ActClass(1000, "Null", is_ignore=True),
-))
